@@ -122,8 +122,8 @@ LogObject::LogObject(LogConfig *cfg, const LogFormat *format, const char *log_di
   // compute_signature is a static function
   m_signature = compute_signature(m_format, m_basename, m_flags);
 
-  m_logFile =
-    new LogFile(m_filename, header, file_format, m_signature, cfg->ascii_buffer_size, cfg->max_line_size, m_pipe_buffer_size);
+  m_logFile = new LogFile(m_filename, header, file_format, m_signature, cfg->ascii_buffer_size, cfg->max_line_size,
+                          m_pipe_buffer_size, format->escape_type());
 
   if (m_reopen_after_rolling) {
     m_logFile->open_file();
@@ -155,11 +155,13 @@ LogObject::~LogObject()
 //
 // This function generates an object filename according to the following rules:
 //
-// 1.- if no extension is given, add .log for ascii logs, and .blog for
+// 1.- 'stdout' and 'stderr' are treated as special strings indicating file
+//     descriptors for the stdout and stderr streams.
+// 2.- if no extension is given, add .log for ascii logs, and .blog for
 //     binary logs
-// 2.- if an extension is given, then do not modify filename and use that
+// 3.- if an extension is given, then do not modify filename and use that
 //     extension regardless of type of log
-// 3.- if there is a '.' at the end of the name, then do not add an extension
+// 4.- if there is a '.' at the end of the name, then do not add an extension
 //     and remove the '.'. To have a dot at the end of the filename, specify
 //     two ('..').
 //
@@ -168,6 +170,18 @@ LogObject::generate_filenames(const char *log_dir, const char *basename, LogFile
 {
   ink_assert(log_dir && basename);
 
+  std::string_view basename_sv{basename};
+  if (basename_sv == "stdout" || basename_sv == "stderr") {
+    int buffer_size = basename_sv.length() + 1; // Include the null terminator.
+
+    m_filename = static_cast<char *>(ats_malloc(buffer_size));
+    m_basename = static_cast<char *>(ats_malloc(buffer_size));
+
+    strncpy(m_filename, basename, buffer_size);
+    strncpy(m_basename, basename, buffer_size);
+    return;
+  }
+
   int i = -1, len = 0;
   char c;
   while (c = basename[len], c != 0) {
@@ -242,18 +256,6 @@ LogObject::add_filter(LogFilter *filter, bool copy)
   m_filter_list.add(filter, copy);
 }
 
-void
-LogObject::set_filter_list(const LogFilterList &list, bool copy)
-{
-  LogFilter *f;
-
-  m_filter_list.clear();
-  for (f = list.first(); f != nullptr; f = list.next(f)) {
-    m_filter_list.add(f, copy);
-  }
-  m_filter_list.set_conjunction(list.does_conjunction());
-}
-
 // we compute the object signature from the fieldlist_str and the printf_str
 // of the LogFormat rather than from the format_str because the format_str
 // is not part of a LogBuffer header
@@ -261,6 +263,10 @@ LogObject::set_filter_list(const LogFilterList &list, bool copy)
 uint64_t
 LogObject::compute_signature(LogFormat *format, char *filename, unsigned int flags)
 {
+  std::string_view filename_sv{filename};
+  if (filename_sv == "stdout" || filename_sv == "stderr") {
+    return 0;
+  }
   char *fl           = format->fieldlist();
   char *ps           = format->printf_str();
   uint64_t signature = 0;
@@ -550,9 +556,9 @@ LogObject::log(LogAccess *lad, std::string_view text_entry)
   buffer = _checkout_write(&offset, bytes_needed);
 
   if (!buffer) {
-    Note("Skipping the current log entry for %s because its size (%zu) exceeds "
-         "the maximum payload space in a log buffer",
-         m_basename, bytes_needed);
+    SiteThrottledNote("Skipping the current log entry for %s because its size (%zu) exceeds "
+                      "the maximum payload space in a log buffer",
+                      m_basename, bytes_needed);
     return Log::FAIL;
   }
   //
@@ -983,14 +989,17 @@ LogObjectManager::_filename_resolution_abort(const char *filename)
 }
 
 bool
-LogObjectManager::_has_internal_filename_conflict(const char *filename, LogObjectList &objects)
+LogObjectManager::_has_internal_filename_conflict(std::string_view filename, LogObjectList &objects)
 {
+  if (filename == "stdout" || filename == "stderr") {
+    return false;
+  }
   for (auto &object : objects) {
     // an internal conflict exists if two objects request the
     // same filename, regardless of the object signatures, since
     // two objects writing to the same file would produce a
     // log with duplicate entries and non monotonic timestamps
-    if (strcmp(object->get_full_filename(), filename) == 0) {
+    if (filename == object->get_full_filename()) {
       return true;
     }
   }
@@ -1119,8 +1128,6 @@ LogObjectManager::open_local_pipes()
 void
 LogObjectManager::transfer_objects(LogObjectManager &old_mgr)
 {
-  unsigned num_kept_objects = 0;
-
   Debug("log-config-transfer", "transferring objects from LogObjectManager %p, to %p", &old_mgr, this);
 
   if (is_debug_tag_set("log-config-transfer")) {
@@ -1163,7 +1170,6 @@ LogObjectManager::transfer_objects(LogObjectManager &old_mgr)
         if (new_obj->refcount_dec() == 0) {
           delete new_obj;
         }
-        ++num_kept_objects;
         break;
       }
     }
